@@ -48,7 +48,7 @@ typedef struct _VideoApp
 
   ClutterActor *control_seek1, *control_seek2, *control_seekbar;
 
-  gboolean      controls_showing, paused;
+  gboolean      controls_showing, paused, mouse_in_window;
 
   guint         controls_timeout;
 } VideoApp;
@@ -88,6 +88,29 @@ controls_timeout_cb (gpointer data)
   return FALSE;
 }
 
+static ClutterTimeline *
+_actor_animate (ClutterActor         *actor,
+                ClutterAnimationMode  mode,
+                guint                 duration,
+                const gchar          *first_property,
+                ...)
+{
+  va_list args;
+
+  clutter_actor_save_easing_state (actor);
+  clutter_actor_set_easing_mode (actor, mode);
+  clutter_actor_set_easing_duration (actor, duration);
+
+  va_start (args, first_property);
+  g_object_set_valist (G_OBJECT (actor), first_property, args);
+  va_end (args);
+
+  clutter_actor_restore_easing_state (actor);
+
+  return CLUTTER_TIMELINE (clutter_actor_get_transition (actor,
+                                                         first_property));
+}
+
 static void
 show_controls (VideoApp *app, gboolean vis)
 {
@@ -110,9 +133,9 @@ show_controls (VideoApp *app, gboolean vis)
       app->controls_showing = TRUE;
 
       clutter_stage_show_cursor (CLUTTER_STAGE (app->stage));
-      clutter_actor_animate (app->control, CLUTTER_EASE_OUT_QUINT, 250,
-                             "opacity", 224,
-                             NULL);
+      _actor_animate (app->control, CLUTTER_EASE_OUT_QUINT, 250,
+                      "opacity", 224,
+                      NULL);
 
       return;
     }
@@ -121,10 +144,11 @@ show_controls (VideoApp *app, gboolean vis)
     {
       app->controls_showing = FALSE;
 
-      clutter_stage_hide_cursor (CLUTTER_STAGE (app->stage));
-      clutter_actor_animate (app->control, CLUTTER_EASE_OUT_QUINT, 250,
-                             "opacity", 0,
-                             NULL);
+      if (app->mouse_in_window)
+        clutter_stage_hide_cursor (CLUTTER_STAGE (app->stage));
+      _actor_animate (app->control, CLUTTER_EASE_OUT_QUINT, 250,
+                      "opacity", 0,
+                      NULL);
       return;
     }
 }
@@ -154,11 +178,11 @@ toggle_pause_state (VideoApp *app)
 }
 
 static void
-reset_animation (ClutterAnimation *animation,
+reset_animation (ClutterTimeline  *animation,
                  VideoApp         *app)
 {
   if (app->vtexture)
-    clutter_actor_set_rotation (app->vtexture, CLUTTER_Y_AXIS, 0.0, 0, 0, 0);
+    clutter_actor_set_rotation_angle (app->vtexture, CLUTTER_Y_AXIS, 0.0);
 }
 
 static gboolean
@@ -215,22 +239,19 @@ input_cb (ClutterStage *stage,
 
     case CLUTTER_KEY_PRESS:
       {
-        ClutterVertex center = { 0, };
-        ClutterAnimation *animation = NULL;
+        ClutterTimeline *animation = NULL;
 
         switch (clutter_event_get_key_symbol (event))
           {
           case CLUTTER_d:
             if (app->vtexture)
               {
-                clutter_container_remove_actor (CLUTTER_CONTAINER (app->stage),
-                                                app->vtexture);
+                clutter_actor_remove_child (app->stage, app->vtexture);
                 app->vtexture = NULL;
               }
             if (app->control)
               {
-                clutter_container_remove_actor (CLUTTER_CONTAINER (app->stage),
-                                                app->control);
+                clutter_actor_remove_child (app->stage, app->control);
                 app->control = NULL;
               }
             break;
@@ -243,16 +264,14 @@ input_cb (ClutterStage *stage,
             if (app->vtexture == NULL)
               break;
 
-            center.x = clutter_actor_get_width (app->vtexture) / 2;
+            clutter_actor_set_pivot_point (app->vtexture, 0.5, 0);
+
+            animation = _actor_animate (app->vtexture, CLUTTER_LINEAR, 500,
+                                        "rotation-angle-y", 360.0, NULL);
 
-            animation =
-              clutter_actor_animate (app->vtexture, CLUTTER_LINEAR, 500,
-                                     "rotation-angle-y", 360.0,
-                                     "fixed::rotation-center-y", &center,
-                                     NULL);
             g_signal_connect_after (animation, "completed",
-                              G_CALLBACK (reset_animation),
-                              app);
+                                    G_CALLBACK (reset_animation),
+                                    app);
             handled = TRUE;
             break;
 
@@ -262,6 +281,17 @@ input_cb (ClutterStage *stage,
             break;
           }
       }
+
+    case CLUTTER_ENTER:
+      app->mouse_in_window = TRUE;
+      g_object_set (app->stage, "cursor-visible", app->controls_showing, NULL);
+      break;
+
+    case CLUTTER_LEAVE:
+      app->mouse_in_window = FALSE;
+      clutter_stage_show_cursor (CLUTTER_STAGE (app->stage));
+      break;
+
     default:
       break;
     }
@@ -357,6 +387,15 @@ on_video_texture_eos (ClutterMedia *media,
     }
 }
 
+static ClutterActor *
+_new_rectangle_with_color (ClutterColor *color)
+{
+  ClutterActor *actor = clutter_actor_new ();
+  clutter_actor_set_background_color (actor, color);
+
+  return actor;
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -395,7 +434,7 @@ main (int argc, char *argv[])
     }
 
   stage = clutter_stage_new ();
-  clutter_stage_set_color (CLUTTER_STAGE (stage), &stage_color);
+  clutter_actor_set_background_color (stage, &stage_color);
   clutter_actor_set_size (stage, 768, 576);
   clutter_stage_set_minimum_size (CLUTTER_STAGE (stage), 640, 480);
   if (opt_fullscreen)
@@ -471,7 +510,7 @@ main (int argc, char *argv[])
                 NULL);
 
   /* Create the control UI */
-  app->control = clutter_group_new ();
+  app->control = clutter_actor_new ();
 
   app->control_bg =
     clutter_texture_new_from_file ("vid-panel.png", NULL);
@@ -482,9 +521,9 @@ main (int argc, char *argv[])
 
   g_assert (app->control_bg && app->control_play && app->control_pause);
 
-  app->control_seek1   = clutter_rectangle_new_with_color (&control_color1);
-  app->control_seek2   = clutter_rectangle_new_with_color (&control_color2);
-  app->control_seekbar = clutter_rectangle_new_with_color (&control_color1);
+  app->control_seek1   = _new_rectangle_with_color (&control_color1);
+  app->control_seek2   = _new_rectangle_with_color (&control_color2);
+  app->control_seekbar = _new_rectangle_with_color (&control_color1);
   clutter_actor_set_opacity (app->control_seekbar, 0x99);
 
   app->control_label =
@@ -494,15 +533,13 @@ main (int argc, char *argv[])
 
   clutter_actor_hide (app->control_play);
 
-  clutter_container_add (CLUTTER_CONTAINER (app->control),
-                         app->control_bg,
-                         app->control_play,
-                         app->control_pause,
-                         app->control_seek1,
-                         app->control_seek2,
-                         app->control_seekbar,
-                         app->control_label,
-                         NULL);
+  clutter_actor_add_child (app->control, app->control_bg);
+  clutter_actor_add_child (app->control, app->control_play);
+  clutter_actor_add_child (app->control, app->control_pause);
+  clutter_actor_add_child (app->control, app->control_seek1);
+  clutter_actor_add_child (app->control, app->control_seek2);
+  clutter_actor_add_child (app->control, app->control_seekbar);
+  clutter_actor_add_child (app->control, app->control_label);
 
   clutter_actor_set_opacity (app->control, 0xee);
 
@@ -519,17 +556,15 @@ main (int argc, char *argv[])
   clutter_actor_set_position (app->control_label, 82, 29);
 
   /* Add control UI to stage */
-  clutter_container_add (CLUTTER_CONTAINER (stage),
-                         app->vtexture,
-                         app->control,
-                         NULL);
+  clutter_actor_add_child (stage, app->vtexture);
+  clutter_actor_add_child (stage, app->control);
 
   position_controls (app, app->control);
 
   clutter_stage_hide_cursor (CLUTTER_STAGE (stage));
-  clutter_actor_animate (app->control, CLUTTER_EASE_OUT_QUINT, 1000,
-                         "opacity", 0,
-                         NULL);
+  _actor_animate (app->control, CLUTTER_EASE_OUT_QUINT, 1000,
+                  "opacity", 0,
+                  NULL);
 
   /* Hook up other events */
   g_signal_connect (stage, "event", G_CALLBACK (input_cb), app);
